//------------------------------------------------------------------
#property copyright "mladen"
#property link      "www.forex-tsd.com"
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  LimeGreen
#property indicator_color2  Orange
#property indicator_color3  Orange
#property indicator_color4  Silver//Red
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_style4  STYLE_DOT
#property indicator_level1  0
#property strict

//
//
//
//
//

enum prices
{
   pr_close,   // Close
   pr_open,    // Open
   pr_high,    // High
   pr_low,     // Low
   pr_median,  // Median
   pr_typical, // Typical
   pr_weighted // Weighted
};
extern string TimeFrame = "Current time frame";  // Time frame to use
extern int    FastEMA   = 12;                    // Fast ema period
extern int    SlowEMA   = 26;                    // Slow ema period
extern int    SignalEMA =  9;                    // Signal ema period
extern prices Price     = pr_close;              // Price to use for MACD
extern bool   ColorOnSignalCross = true;         // Change colors on signal line crosses?
extern bool   DoubleMacd = true;                 // Should it be double macd?

double macd[];
double macdDa[];
double macdDb[];
double signal[];
double colors[];
string indicatorFileName;
bool   returnBars;
int    timeFrame;


//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,macd);
   SetIndexBuffer(1,macdDa);
   SetIndexBuffer(2,macdDb);
   SetIndexBuffer(3,signal);
   SetIndexBuffer(4,colors);
      string add = ""; if (DoubleMacd) add = "double";
      
         //
         //
         //
         //
         //
         
         indicatorFileName = WindowExpertName();
         returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
         timeFrame         = stringToTimeFrame(TimeFrame);
         
         //
         //
         //
         //
         //
         
      IndicatorShortName(timeFrameToString(timeFrame)+" MACD "+add+" ("+(string)FastEMA+","+(string)SlowEMA+","+(string)SignalEMA+")");
   return(0);
}
int deinit()
{
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { macd[0] = limit+1; return(0); }
         if (timeFrame != Period())
         {
            limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
            if (colors[limit]==-1) CleanPoint(limit,macdDa,macdDb);
            for(int i=limit; i>=0; i--)
            {
               int y = iBarShift(NULL,timeFrame,Time[i]);               
                  macd[i]   = iCustom(NULL,timeFrame,indicatorFileName,"",FastEMA,SlowEMA,SignalEMA,Price,ColorOnSignalCross,DoubleMacd,0,y);
                  signal[i] = iCustom(NULL,timeFrame,indicatorFileName,"",FastEMA,SlowEMA,SignalEMA,Price,ColorOnSignalCross,DoubleMacd,3,y);
                  colors[i] = iCustom(NULL,timeFrame,indicatorFileName,"",FastEMA,SlowEMA,SignalEMA,Price,ColorOnSignalCross,DoubleMacd,4,y);
                  macdDa[i] = EMPTY_VALUE;
                  macdDb[i] = EMPTY_VALUE;
                  if (y==iBarShift(NULL,timeFrame,Time[(int)MathMax(i-1,0)])) continue;

                  //
                  //
                  //
                  //
                  //

                  int n,m; datetime time = iTime(NULL,timeFrame,y);
                     for(n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
                     for(m = 1; m < n && (i+n)<Bars && (i+m)<Bars; m++)
                     {
                        macd[i+m]   = macd[i]   + (macd[i+n]   - macd[i]  ) * m/n;
                        signal[i+m] = signal[i] + (signal[i+n] - signal[i]) * m/n;
                     }               
            }
            for(int i = limit; i >= 0 ; i--) if (colors[i]==-1) PlotPoint(i,macdDa,macdDb,macd);
            return(0);
         }

   //
   //
   //
   //
   //
   
   if (colors[limit]==-1) CleanPoint(limit,macdDa,macdDb);
   for(int i = limit; i >= 0 ; i--)
   {
      double price = iMA(NULL,0,1,0,MODE_SMA,(int)Price,i);
      double tmacd = iEma(price,FastEMA,i,0)-iEma(price,SlowEMA,i,1);
      if (DoubleMacd)
            macd[i] = iEma(tmacd,FastEMA,i,2)-iEma(tmacd,SlowEMA,i,3); 
      else  macd[i] = tmacd;            
      macdDa[i] = EMPTY_VALUE;
      macdDb[i] = EMPTY_VALUE;
      signal[i] = iEma(macd[i],SignalEMA,i,4);
      if (i<Bars-1)
      {
         colors[i] = colors[i+1];
         if (ColorOnSignalCross)
         {
            if (macd[i]>signal[i]) colors[i] =  1;
            if (macd[i]<signal[i]) colors[i] = -1;
         }
         else
         {
            if (macd[i]>macd[i+1]) colors[i] =  1;
            if (macd[i]<macd[i+1]) colors[i] = -1;
         }
         if (colors[i]==-1) PlotPoint(i,macdDa,macdDb,macd);
      }         
   }
   return(0);
}


//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

double workEma[][5];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r = Bars-r-1;

   //
   //
   //
   //
   //
   
   if (r==0) workEma[r][instanceNo]=price;
   else
   {
      double alpha = 2.0 / (1.0+period);
             workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   }             
   return(workEma[r][instanceNo]);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-2) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-3) return;
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   StringToUpper(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+(string)iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                              return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}